import requests
import json
import pandas as pd
import hashlib

with open("fred_apikey.txt", "r") as f:
    apikey = f.readline().strip()
series = "SP500"
start = "2019-01-01"
end = "2024-01-01"
url = (f"https://api.stlouisfed.org/fred/series/observations?"
      f"series_id={series}&api_key={apikey}"
      f"&observation_start={start}&observation_end={end}&file_type=json")
response = requests.get(url)
response.raise_for_status()

json_data = response.json()
with open("assignment1/SP500.json", "w") as f:
    f.write(json.dumps(json_data, indent=4))

# start to read the json file
df = pd.json_normalize(json_data, 'observations')
print(df.head())
print(df.dtypes)

# data cleaning and preprocessing
df = df.drop(["realtime_start", "realtime_end"], axis=1)
df = df[df.value != "."] # drop invalid value
df["date"] = pd.to_datetime(df["date"])
df["value"] = pd.to_numeric(df["value"])

# Plot and index
df = df.set_index(["date"])
plt = df.plot(title="SP500(2019-2024)", legend=False)
plt.set_xlabel("Date")
plt.set_ylabel("Value")
# ensured the png file was created successfully on my end 
df.to_csv("assignment1/data/SP500.csv", lineterminator='\n')
plt.get_figure().savefig("assignment1/results/SP500.png")

with open("assignment1/data/SP500.csv", "rb") as f:
    data = f.read()
    sha256hash = hashlib.sha256(data).hexdigest()
    
with open("assignment1/data/SP500.sha", "w") as f:
    f.write(sha256hash)

import datetime
today_date = datetime.date.today().strftime("%Y-%m-%d")

bibtex_entry = f"""@misc{{
    author = "Federal Reserve Economic Data",
    title = "SP 500",
    publisher = "Federal Reserve Economic Data",
    year = "2024",
    note = "Retrieved on {today_date}",
    url = "https://api.stlouisfed.org/fred/series/observations?series_id=SP500&api_key=apikey&file_type=json"
}}"""

with open("assignment1/data_citation.bib", "w") as file:
    file.write(bibtex_entry)

print("Data citation written to assignment1/data_citation.bib.")


